-----------------------
-- Gathering addon name
-----------------------
local AddOnName = ...;
-----------------------

-----------------------
-- Setting up main tables
-----------------------
local TrafficMeter = {};
	TrafficMeter.IsActive = false;
	TrafficMeter.Ticker = nil;
	TrafficMeter.TickerTime = 0.5;
	TrafficMeter.bandwidthIn = 0;
	TrafficMeter.bandwidthOut = 0;
	TrafficMeter.latencyHome = 0;
	TrafficMeter.latencyWorld = 0;

local TrafficMeterGUI = {};
	TrafficMeterGUI.Window = nil;
	TrafficMeterGUI.Height = 20;
	TrafficMeterGUI.Width = 200;
	TrafficMeterGUI.Spacing = 5;
	TrafficMeterGUI.ResolutionY = 0;
	TrafficMeterGUI.BackgroundColour	= {1.0, 1.0, 1.0, 0.0};
	TrafficMeterGUI.TextColourGeneral	= {0.8, 0.8, 0.8, 1.0};
	TrafficMeterGUI.TextColourPingGood	= {0.2, 1.0, 0.2, 1.0};
	TrafficMeterGUI.TextColourPingOK	= {1.0, 1.0, 0.2, 1.0};
	TrafficMeterGUI.TextColourPingBad	= {1.0, 0.2, 0.2, 1.0};
-----------------------

-----------------------
-- Shortening the call of support functions
-----------------------
local Print2 = SupportFunctions.Print2;
local TableDump = SupportFunctions.TableDump;
local TransformDotsToIndexedTable = SupportFunctions.TransformDotsToIndexedTable;
local CutElementsFromTableStart = SupportFunctions.CutElementsFromTableStart;
-----------------------

-----------------------
-- Creating the GUI
-----------------------
function TrafficMeterGUI:CreateGUI()
	-- Creating frame for data display
	TrafficMeterGUI.Window = CreateFrame("Frame", "TrafficMeterWindow", UIParent);
	if(not TrafficMeterGUI.Window) then Print2(TM_Strings["ERROR__UI_CREATION_MAIN"]); return; end

	-- Position
	--TrafficMeterGUI.Window:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
	-- Setting default position causes error in loading config in some cases, frame is positioned with yOffset equal to what was first set here instead of correctly loaded yOffset value.
	-- Setting first position twice in same frame seems to be buggy, no chance to fix that myself.

	-- Dimensions
	TrafficMeterGUI.Window:SetHeight(TrafficMeterGUI.Height);
	TrafficMeterGUI.Window:SetWidth(TrafficMeterGUI.Width);

	-- Background
	TrafficMeterGUI.Window.tex = TrafficMeterGUI.Window:CreateTexture(nil, "ARTWORK");
	TrafficMeterGUI.Window.tex:SetHeight(TrafficMeterGUI.Window:GetHeight());
	TrafficMeterGUI.Window.tex:SetWidth(TrafficMeterGUI.Window:GetWidth());
	TrafficMeterGUI.Window.tex:SetPoint("CENTER", TrafficMeterGUI.Window, "CENTER");
	TrafficMeterGUI.Window.tex:SetColorTexture(unpack(TrafficMeterGUI.BackgroundColour));

	-- Window movement stuff
	TrafficMeterGUI.Window.isMoving = false;
	TrafficMeterGUI.Window:SetMovable(true);
	TrafficMeterGUI.Window:EnableMouse(true);
	TrafficMeterGUI.Window:SetScript("OnMouseUp", TrafficMeter.OnMouseUp);
	TrafficMeterGUI.Window:SetScript("OnMouseDown", TrafficMeter.OnMouseDown);

	-- Label - Incoming traffic
	TrafficMeterGUI.Window.labelIn = TrafficMeterGUI.Window:CreateFontString("LabelIn", "OVERLAY");
	TrafficMeterGUI.Window.labelIn:SetPoint("LEFT", TrafficMeterGUI.Window, "LEFT", 0, 0);
	TrafficMeterGUI.Window.labelIn:SetFont("Fonts\\FRIZQT__.TTF", TrafficMeterGUI.Height * 0.6, "OUTLINE");
	TrafficMeterGUI.Window.labelIn:SetJustifyH("CENTER");
	TrafficMeterGUI.Window.labelIn:SetShadowOffset(1, -1);
	TrafficMeterGUI.Window.labelIn:SetTextColor(unpack(TrafficMeterGUI.TextColourGeneral));
	TrafficMeterGUI.Window.labelIn:SetText("");

	-- Label - Outgoing traffic
	TrafficMeterGUI.Window.labelOut = TrafficMeterGUI.Window:CreateFontString("LabelOut", "OVERLAY");
	TrafficMeterGUI.Window.labelOut:SetPoint("LEFT", TrafficMeterGUI.Window.labelIn, "RIGHT", TrafficMeterGUI.Spacing, 0);
	TrafficMeterGUI.Window.labelOut:SetFont("Fonts\\FRIZQT__.TTF", TrafficMeterGUI.Height * 0.6, "OUTLINE");
	TrafficMeterGUI.Window.labelOut:SetJustifyH("CENTER");
	TrafficMeterGUI.Window.labelOut:SetShadowOffset(1, -1);
	TrafficMeterGUI.Window.labelOut:SetTextColor(unpack(TrafficMeterGUI.TextColourGeneral));
	TrafficMeterGUI.Window.labelOut:SetText("");

	-- Label - Ping Home
	TrafficMeterGUI.Window.labelPingHome = TrafficMeterGUI.Window:CreateFontString("LabelPingHome", "OVERLAY");
	TrafficMeterGUI.Window.labelPingHome:SetPoint("LEFT", TrafficMeterGUI.Window.labelOut, "RIGHT", TrafficMeterGUI.Spacing, 0);
	TrafficMeterGUI.Window.labelPingHome:SetFont("Fonts\\FRIZQT__.TTF", TrafficMeterGUI.Height * 0.6, "OUTLINE");
	TrafficMeterGUI.Window.labelPingHome:SetJustifyH("CENTER");
	TrafficMeterGUI.Window.labelPingHome:SetShadowOffset(1, -1);
	TrafficMeterGUI.Window.labelPingHome:SetTextColor(unpack(TrafficMeterGUI.TextColourGeneral));
	TrafficMeterGUI.Window.labelPingHome:SetText("");

	-- Label - Ping World
	TrafficMeterGUI.Window.labelPingWorld = TrafficMeterGUI.Window:CreateFontString("LabelPingWorld", "OVERLAY");
	TrafficMeterGUI.Window.labelPingWorld:SetPoint("LEFT", TrafficMeterGUI.Window.labelPingHome, "RIGHT", TrafficMeterGUI.Spacing, 0);
	TrafficMeterGUI.Window.labelPingWorld:SetFont("Fonts\\FRIZQT__.TTF", TrafficMeterGUI.Height * 0.6, "OUTLINE");
	TrafficMeterGUI.Window.labelPingWorld:SetJustifyH("CENTER");
	TrafficMeterGUI.Window.labelPingWorld:SetShadowOffset(1, -1);
	TrafficMeterGUI.Window.labelPingWorld:SetTextColor(unpack(TrafficMeterGUI.TextColourGeneral));
	TrafficMeterGUI.Window.labelPingWorld:SetText("");

	-- Readying
	TrafficMeter.IsActive = true;
end
-----------------------

-----------------------
-- Loads the UI config
-----------------------
function TrafficMeter:LoadConfig()
	if(TrafficMeterConfig) then
		TrafficMeter:SetPosition(TrafficMeterGUI.Window);
	else
		TrafficMeterConfig = {};
		TrafficMeterConfig.WindowHook = "CENTER";
		TrafficMeterConfig.ParentHook = "CENTER";
		TrafficMeterConfig.xOffset = 0;
		TrafficMeterConfig.yOffset = 0;

		TrafficMeter:SetPosition(TrafficMeterGUI.Window);
	end
end
-----------------------

-----------------------
-- Handling the UI mouse up event
-----------------------
function TrafficMeter:OnMouseUp(button)
	if(button == "LeftButton" and TrafficMeterGUI.Window.isMoving) then
		TrafficMeterGUI.Window:SetMovable(false);
		TrafficMeterGUI.Window:StopMovingOrSizing();
		TrafficMeterGUI.Window.isMoving = false;
		TrafficMeter:GetPosition(TrafficMeterGUI.Window);
	end
end
-----------------------

-----------------------
-- Handling the UI mouse down event
-----------------------
function TrafficMeter:OnMouseDown(button)
	if(button == "LeftButton" and not TrafficMeterGUI.Window.isMoving) then
		TrafficMeterGUI.Window:SetMovable(true);
		TrafficMeterGUI.Window:StartMoving();
		TrafficMeterGUI.Window.isMoving = true;
	end
end
-----------------------

-----------------------
-- Gets and saves the current main window position
-----------------------
function TrafficMeter:GetPosition(frame)
	TrafficMeterConfig.WindowHook, _, TrafficMeterConfig.ParentHook, TrafficMeterConfig.xOffset, TrafficMeterConfig.yOffset = frame:GetPoint();
	TrafficMeterConfig.xOffset = floor(TrafficMeterConfig.xOffset + 0.5);
	TrafficMeterConfig.yOffset = floor(TrafficMeterConfig.yOffset + 0.5);
end
-----------------------

-----------------------
-- Loads and sets the main window position
-----------------------
function TrafficMeter:SetPosition(frame)
	if(
	(not TrafficMeterConfig.WindowHook) or
	(not TrafficMeterConfig.ParentHook) or
	(not TrafficMeterConfig.xOffset) or
	(not TrafficMeterConfig.yOffset)) then
		Print2(TM_Strings["ERROR__INVALID_CONFIG"]);
		TrafficMeterConfig = nil;
		TrafficMeter:LoadConfig();
	end
	frame:SetPoint(TrafficMeterConfig.WindowHook, UIParent, TrafficMeterConfig.ParentHook, TrafficMeterConfig.xOffset, TrafficMeterConfig.yOffset);
end
-----------------------

-----------------------
-- Updates stored traffic data
-----------------------
function TrafficMeter:UpdateData()
	local tempColour = {};

	-- Checking that addon is fully initialised
	if(not TrafficMeter.IsActive) then return; end

	-- Updating traffic values
	TrafficMeter.bandwidthIn, TrafficMeter.bandwidthOut, TrafficMeter.latencyHome, TrafficMeter.latencyWorld = GetNetStats();
	if(TrafficMeter.bandwidthIn < 1) then
		TrafficMeterGUI.Window.labelIn:SetText(string.format(TM_Strings["INFO__IN_BYTES"], TrafficMeter.bandwidthIn * 1024));
	else
		TrafficMeterGUI.Window.labelIn:SetText(string.format(TM_Strings["INFO__IN_KILOBYTES"], TrafficMeter.bandwidthIn));
	end
	if(TrafficMeter.bandwidthOut < 1) then
		TrafficMeterGUI.Window.labelOut:SetText(string.format(TM_Strings["INFO__OUT_BYTES"], TrafficMeter.bandwidthOut * 1024));
	else
		TrafficMeterGUI.Window.labelOut:SetText(string.format(TM_Strings["INFO__OUT_KILOBYTES"], TrafficMeter.bandwidthOut));
	end

	-- Updating ping colours and values
	-- -- Home
	if(TrafficMeter.latencyHome > 299) then
		tempColour = TrafficMeterGUI.TextColourPingBad;
	elseif(TrafficMeter.latencyHome > 99) then
		tempColour = TrafficMeterGUI.TextColourPingOK;
	else
		tempColour = TrafficMeterGUI.TextColourPingGood;
	end
	TrafficMeterGUI.Window.labelPingHome:SetTextColor(unpack(tempColour));
	TrafficMeterGUI.Window.labelPingHome:SetText(string.format(TM_Strings["INFO__CHAT"], TrafficMeter.latencyHome));

	-- -- World
	if(TrafficMeter.latencyWorld > 299) then
		tempColour = TrafficMeterGUI.TextColourPingBad;
	elseif(TrafficMeter.latencyWorld > 99) then
		tempColour = TrafficMeterGUI.TextColourPingOK;
	else
		tempColour = TrafficMeterGUI.TextColourPingGood;
	end
	TrafficMeterGUI.Window.labelPingWorld:SetTextColor(unpack(tempColour));
	TrafficMeterGUI.Window.labelPingWorld:SetText(string.format(TM_Strings["INFO__GAME"], TrafficMeter.latencyWorld));

	-- Updating main window width (needed for dragging functionality)
	TrafficMeterGUI.Width = TrafficMeterGUI.Window.labelIn:GetWidth() + TrafficMeterGUI.Window.labelOut:GetWidth() + TrafficMeterGUI.Window.labelPingHome:GetWidth() + TrafficMeterGUI.Window.labelPingWorld:GetWidth() + (3 * TrafficMeterGUI.Spacing);
	TrafficMeterGUI.Window:SetWidth(TrafficMeterGUI.Width);
	TrafficMeterGUI.Window.tex:SetWidth(TrafficMeterGUI.Width);
end
-----------------------

-----------------------
-- Initialisation method
-----------------------
function TrafficMeter:Init()
	TrafficMeter.Ticker = C_Timer.NewTicker(TrafficMeter.TickerTime, TrafficMeter.UpdateData);
	TrafficMeterGUI:CreateGUI();
	TrafficMeter:LoadConfig();
end
-----------------------

-----------------------
-- Handling the ADDON_LOADED event
-----------------------
local function OnADDON_LOADED(givenAddOnName)
	if(givenAddOnName == AddOnName) then
		TrafficMeter:Init();
	end
end
-----------------------

-----------------------
-- General event handling method
-----------------------
local function OnEvent(...)
	local args = TransformDotsToIndexedTable(...);
	local event = args[2];

	args = CutElementsFromTableStart(args, 2); -- Cutting off self reference and event type

	if(event == "ADDON_LOADED") then OnADDON_LOADED(unpack(args));
	else Print2(string.format(TM_Strings["ERROR__UNHANDLED_EVENT"], tostring(event), tostring(AddOnName)));
	end
end
-----------------------

local events = CreateFrame("Frame");
events:SetScript("OnEvent", OnEvent);
events:RegisterEvent("ADDON_LOADED");